@@ -14,14 +14,17 @@ program draw
   integer :: nx, ny, nl
   integer :: nargv, fig_type
   integer :: xdmin, xdmax, ydmin, ydmax
+  integer, dimension(2) :: ind 
   real :: xmin, xmax, ymin, ymax, xscale, yscale
   real, allocatable, dimension(:) :: x, y
   real, allocatable, dimension(:,:) :: val1, val2, val3, val4, xd, yd
+  real, allocatable, dimension(:,:) :: dum1, dum2
   character(80) :: argv
   character(80) :: cont_val, shade_val, vx_val, vy_val, fname, txtname
   character(80) :: x_axis, y_axis
   character(80), allocatable, dimension(:,:) :: cval
   character(80) :: draw_title, title
+  character(1) :: line_flag
   real :: cmin, cmax, smin, smax, sfact, cfact, vxfact, vyfact
 
   cont_val=''
@@ -113,6 +116,7 @@ program draw
         xd(i,1)=c2r_convert( cval(1,i) )
         yd(i,1)=c2r_convert( cval(2,i) )
      end do
+     ind=(/8,8/)  ! index define
   end if
 
 !-- ΰη
@@ -121,13 +125,13 @@ program draw
   call nearest_search_1d( y, ymin, ydmin )
   call nearest_search_1d( y, ymax, ydmax )
 
-  xscale=0.005
+  xscale=0.025
   call calc_vscale( (/(xmax-xmin), (ymax-ymin)/), (/0.6, 0.6/), xscale, yscale )
-  yscale=yscale
+!  yscale=yscale
 
 !  dummy=-10.0
 write(*,*) "draw", trim(draw_title)
-  call color_setting( 10, (/smin, smax/), min_tab=999, max_tab=1999 )  
+  call color_setting( 10, (/smin, smax/), min_tab=10999, max_tab=99999 )  
 
   call DclOpenGraphics(2)
 
@@ -140,7 +144,7 @@ write(*,*) "draw", trim(draw_title)
   &       (/cmin, cmax/), (/smin, smax/),  &
   &       (/trim(x_axis), trim(y_axis)/), (/'(f6.1)', 'f(6.1)'/),  &
   &       c_num=(/10, 10/), viewx_int=(/0.2, 0.8/), viewy_int=(/0.2, 0.8/),  &
-  &       xg=xd(:,1:1), yg=yd(:,1:1), trigleg='a' )  !,  &
+  &       xg=xd(:,1:1), yg=yd(:,1:1), trigleg='a', l_idx=ind )  !,  &
      else
      call Dcl_2D_cont_shade( trim(draw_title),  &
   &       x(xdmin:xdmax), y(ydmin:ydmax),  &
@@ -149,9 +153,11 @@ write(*,*) "draw", trim(draw_title)
   &       (/cmin, cmax/), (/smin, smax/),  &
   &       (/trim(x_axis), trim(y_axis)/), (/'(f6.1)', 'f(6.1)'/),  &
   &       c_num=(/10, 10/), viewx_int=(/0.2, 0.8/), viewy_int=(/0.2, 0.8/),  &
-  &       trigleg='a' )  !,  &
+  &       trigleg='a', l_idx=ind )  !,  &
      end if
-  else if(fig_type==2)then
+  end if
+
+  if(fig_type==2)then
      if(txtname/='')then
      call Dcl_2D_cont_shade_vec( trim(draw_title),  &
   &       x(xdmin:xdmax), y(ydmin:ydmax),  &
@@ -162,7 +168,7 @@ write(*,*) "draw", trim(draw_title)
   &       (/20, 20/), (/cmin, cmax/), (/smin, smax/),  &
   &       (/trim(x_axis), trim(y_axis)/), (/'(f6.1)', 'f(6.1)'/),  &
   &       c_num=(/10, 10/), viewx_int=(/0.2, 0.8/), viewy_int=(/0.2, 0.8/),  &
-  &       xg=xd(:,1:1), yg=yd(:,1:1), trigleg='a',  &
+  &       xg=xd(:,1:1), yg=yd(:,1:1), trigleg='a', l_idx=ind,  &
   &       unitv=.true., vfact=(/xscale, yscale/), unit_fact_sign=.true.,  &
   &       unit_fact=(/vxfact, vyfact/), unit_title=(/'U','V'/) )
      else
@@ -175,12 +181,85 @@ write(*,*) "draw", trim(draw_title)
   &       (/20, 20/), (/cmin, cmax/), (/smin, smax/),  &
   &       (/trim(x_axis), trim(y_axis)/), (/'(f6.1)', 'f(6.1)'/),  &
   &       c_num=(/10, 10/), viewx_int=(/0.2, 0.8/), viewy_int=(/0.2, 0.8/),  &
-  &       trigleg='a',  &
+  &       trigleg='a', l_idx=ind,  &
   &       unitv=.true., vfact=(/xscale, yscale/), unit_fact_sign=.true.,  &
   &       unit_fact=(/vxfact, vyfact/), unit_title=(/'U','V'/) )
      end if
   end if
 
+  if(fig_type<0)then
+     if(trim(cont_val)/='')then
+        if(trim(shade_val)/='')then
+           line_flag = 'a'
+        else
+           line_flag = 'l'
+        end if
+     else
+        if(trim(shade_val)/='')then
+           line_flag = 'p'
+        else
+           write(*,*) "ERROR : shade_val or cont_val must set. STOP"
+           stop
+        end if
+     end if
+  end if
+
+  if(fig_type==-1)then  ! x axis line draw
+     allocate(xd(nx,ny))
+     do i=1,nx
+        xd(i,:)=x(i)
+     end do
+
+     ! ΰϰΤ, c  s 礭 cmin, cmax ľ
+     if(cmin>smin)then
+        cmin=smin
+     end if
+     if(cmax<smax)then
+        cmax=smax
+     end if
+
+     call Dcl_PL( trim(line_flag), trim(draw_title),  &
+  &       xd(xdmin:xdmax,ydmin:ydmax),  &
+  &       val1(xdmin:xdmax,ydmin:ydmax),  &
+  &       xd(xdmin:xdmax,ydmin:ydmax),  &
+  &       val2(xdmin:xdmax,ydmin:ydmax),  &
+  &       (/trim(x_axis), '               '/),  &
+  &       x_int=(/xd(xdmin,1), xd(xdmax,1)/), y_int=(/cmin, cmax/),  &
+  &       viewx_int=(/0.2, 0.8/), viewy_int=(/0.2, 0.8/) )
+  else
+     allocate(yd(ny,nx))  ! ν֤ 2 ܤοʤΤny, nxؤ.
+     allocate(dum1(ny,nx))
+     allocate(dum2(ny,nx))
+     do i=1,ny
+        yd(i,:)=y(i)
+     end do
+
+     do j=1,ny
+        do i=1,nx
+           dum1(j,i)=val1(i,j)
+           dum2(j,i)=val2(i,j)
+        end do
+     end do
+
+     ! ΰϰΤ, c  s 礭 cmin, cmax ľ
+     if(cmin>smin)then
+        cmin=smin
+     end if
+     if(cmax<smax)then
+        cmax=smax
+     end if
+
+     call Dcl_PL( trim(line_flag), trim(draw_title),  &
+  &       dum1(xdmin:xdmax,ydmin:ydmax),  &
+  &       yd(ydmin:ydmax,xdmin:xdmax),  &
+  &       dum2(xdmin:xdmax,ydmin:ydmax),  &
+  &       yd(ydmin:ydmax,xdmin:xdmax),  &
+  &       (/'               ', trim(y_axis)/),  &
+  &       x_int=(/yd(ydmin,1), yd(ydmax,1)/), y_int=(/cmin, cmax/),  &
+  &       viewx_int=(/0.2, 0.8/), viewy_int=(/0.2, 0.8/) )
+  end if
+
+
   call DclCloseGraphics
 
 end program
