@@ -817,8 +817,6 @@ subroutine laplacian_1d( x, u, val, undef )
   real, intent(in), optional :: undef
   integer :: i  ! 졼ź
   integer :: nx  ! ǿ
-  real :: scalex(size(x))
-  real :: tmpu(size(x))
 
   nx=size(x)
 
@@ -949,7 +947,6 @@ subroutine turb_visc( signal, x, y, z, u, v, w, rho, nuh, nuv, val, undef, hx, h
               ! Ѥ 1,2,3 ̤˿ľʱ
   character(1) :: signaltau(3)
   integer :: id
-  real, dimension(size(x),size(y),size(z)) :: tmp
   real, dimension(size(x),size(y)) :: stau
   real, dimension(size(x),size(y),size(z)) :: scalex, scaley, scalez
 
@@ -1037,12 +1034,8 @@ subroutine Reynolds_stress( signal, x, y, z, u, v, w, rho, nuh, nuv, val, undef,
   integer :: nx  ! ǿ 1 
   integer :: ny  ! ǿ 2 
   integer :: nz  ! ǿ 3 
-  real, dimension(size(x),size(y),size(z)) :: tau1, tau2, tau3  ! signal 
-              ! Ѥ 1,2,3 ̤˿ľʱ
-  integer :: signaltau
   real :: stau(size(x),size(y))
   real :: sxx(size(x),size(y),size(z)), nu(size(x),size(y),size(z))
-  real, dimension(size(x),size(y),size(z)) :: ddx, ddy, ddz
   real, dimension(size(x),size(y),size(z)) :: scalex, scaley, scalez
 
   nx=size(x)
@@ -1236,8 +1229,6 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
   integer :: nx  ! ǿ 1 
   integer :: ny  ! ǿ 2 
   integer :: nz  ! ǿ 3 
-  real, dimension(size(x),size(y),size(z)) :: tau1, tau2, tau3  ! signal 
-              ! Ѥ 1,2,3 ̤˿ľʱ
   real, dimension(size(x),size(y),size(z)) :: tmpu, tmpv, tmpw
   real, dimension(size(x),size(y),size(z)) :: scalex, scaley, scalez
   real, dimension(size(x),size(y),size(z)) :: ddx, ddy, ddz
@@ -1493,8 +1484,6 @@ subroutine turb_diff( x, y, z, phi, rho, nuh, nuv, val, undef, hx, hy, hz )
               ! Ѥ 1,2,3 ̤˿ľʱ
   character(1) :: signaltau(3)
   integer :: id
-  real, dimension(size(x),size(y),size(z)) :: tmp
-  real, dimension(size(x),size(y),size(z)) :: ddx, ddy, ddz
   real, dimension(size(x),size(y),size(z)) :: scalex, scaley, scalez
 
   signaltau=(/ '1', '2', '3' /)
@@ -1568,7 +1557,6 @@ subroutine Reynolds_scal( signal, x, y, z, phi, rho, nuh, nuv, val, undef, hx, h
   integer :: nx  ! ǿ 1 
   integer :: ny  ! ǿ 2 
   integer :: nz  ! ǿ 3 
-  real, dimension(size(x),size(y),size(z)) :: ddx, ddy, ddz
   real, dimension(size(x),size(y),size(z)) :: scalex, scaley, scalez
 
   nx=size(x)
