@@ -14,8 +14,8 @@ subroutine rectangle_int( x, y, bot, top, res, undeff )  ! 1 ʬ
   real, intent(in) :: y(size(x))  ! ʬؿ
   real, intent(inout) :: res  ! ʬʬ
   real, intent(in), optional :: undeff
-  integer :: i, j, k, nx, i_bot, i_top
-  real :: dx, y_bot, y_top
+  integer :: i, nx, i_bot, i_top
+  real :: y_bot, y_top
 
   nx=size(x)
 
@@ -30,95 +30,107 @@ subroutine rectangle_int( x, y, bot, top, res, undeff )  ! 1 ʬ
 
 !-- ʲʬ
 !-- ºݤˤ, ʬ˺Ƕ᤹
-  do i=1,nx
-     if(x(i)>bot)then
-        if(i==1)then
-           write(*,*) "#### WARNING ####"
-           write(*,*) "there is NOT the bot in the x(i)."  ! ΤȤ, i_bot=1ȤƤƤ, x(1)=bot ʤΤ, ;ʬ׻ϥȤʤƶϤʤ.
-        end if
 
+  if(x(1)>bot)then
+     write(*,*) "#### WARNING ####"
+     write(*,*) "there is NOT the bot in the x(i)."  ! ΤȤ, i_bot=1ȤƤƤ, x(1)=bot ʤΤ, ;ʬ׻ϥȤʤƶϤʤ.
+     i_bot=1
+  end if
+  if(x(nx)<top)then
+     write(*,*) "#### WARNING ####"
+     write(*,*) "there is NOT the top in the x(i)."  ! ΤȤ, i_top=nx ȤƤ, x(nx)=top ʤΤ, ;ʬ׻ϥȤʤ.
+     i_top=nx
+  end if
+
+  do i=2,nx-1
+     if(x(i)>=bot)then  ! i_bot  bot - top κǤ bot ˶Ťʻ
         i_bot=i
         exit
-
      end if
   end do
 
-  do i=1,nx
-     if(x(i)<top)then
-        if(i==nx)then
-           write(*,*) "#### WARNING ####"
-           write(*,*) "there is NOT the top in the x(i)."  ! ΤȤ, i_top=nx ȤƤ, x(nx)=top ʤΤ, ;ʬ׻ϥȤʤ.
-        end if
-
+  do i=nx-1,2,-1
+     if(x(i)<=top)then  ! i_top  bot - top κǤ top ˶Ťʻ
         i_top=i
-
+        exit
      end if
   end do
 
 !-- ʲǳʻҤƤϤޤʤʬޤ䴰
-  y_bot=y(i_bot-1)  &
-  &     +((y(i_bot)-y(i_bot-1))/(x(i_bot)-x(i_bot-1)))*(bot-x(i_bot-1))
-  y_top=y(i_top)  &
-  &     +((y(i_top+1)-y(i_top))/(x(i_top+1)-x(i_top)))*(x(i_top+1)-top)
+  if(present(undeff))then
+     if(i_bot/=1)then
+        if(y(i_bot)/=undeff.and.y(i_bot-1)/=undeff.and.x(i_bot)/=x(i_bot-1))then
+           y_bot=y(i_bot-1)  &
+     &           +((y(i_bot)-y(i_bot-1))/(x(i_bot)-x(i_bot-1)))*(bot-x(i_bot-1))
+        else
+           y_bot=-y(i_bot)  ! Ǹʬǥˤ뤿
+        end if
+     else
+        y_bot=-y(i_bot)  ! Ǹʬǥˤ뤿
+     end if
+     if(i_top/=nx)then
+        if(y(i_top)/=undeff.and.y(i_top+1)/=undeff.and.x(i_top+1)/=x(i_top))then
+           y_top=y(i_top)  &
+     &           +((y(i_top+1)-y(i_top))/(x(i_top+1)-x(i_top)))*(x(i_top+1)-top)
+        else
+           y_top=-y(i_top)
+        end if
+     else
+        y_top=-y(i_top)  ! Ǹʬǥˤ뤿
+     end if
+  else
+     if(i_bot/=1)then
+        y_bot=y(i_bot-1)  &
+     &        +((y(i_bot)-y(i_bot-1))/(x(i_bot)-x(i_bot-1)))*(bot-x(i_bot-1))
+     else
+        y_bot=-y(i_bot)  ! Ǹʬǥˤ뤿
+     end if
+     if(i_top/=nx)then
+        y_top=y(i_top)  &
+     &        +((y(i_top+1)-y(i_top))/(x(i_top+1)-x(i_top)))*(x(i_top+1)-top)
+     else
+        y_top=-y(i_top)  ! Ǹʬǥˤ뤿
+     end if
+  end if
 
-  if(i_bot<=i_top)then  ! ʬ˳ʻҤ 1 İʾ夢Ȥ
+  if(i_bot<i_top)then  ! ʬ˳ʻҤ 2 İʾ夢Ȥ
 
      if(present(undeff))then
-        do i=i_bot,i_top
+        do i=i_bot,i_top-1
            if(y(i+1)/=undeff.and.y(i)/=undeff)then
               if(i==i_bot)then
-                 if(i_bot<i_top)then  ! ʬ֤ 2 ʻҰʾ夢
-                    res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
-  &                     +0.5*(x(i+1)-x(i))*(y(i+1)+y(i))
-                        ! ü;̾ûʬ
-                 else
-                    if(i_bot==i_top)then
-                      ! ʬ֤ 1 ʻҤʤ, ʬԤä, exit ȴ.
-                       res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
-  &                        +0.5*(top-x(i))*(y_top+y(i))
-                       exit
-                    end if
-                 end if
-              else
-                 if(i==i_top)then
-                    res=res+0.5*(top-x(i))*(y_top+y(i))  ! ü;Τ
-                 else
-                    res=res+0.5*(x(i+1)-x(i))*(y(i+1)+y(i))  ! ̾û
-                 end if
-              end if
-           end if
-        end do
-     else
-        do i=i_bot,i_top
-           if(i==i_bot)then
-              if(i_bot<i_top)then  ! ʬ֤ 2 ʻҰʾ夢
                  res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
   &                  +0.5*(x(i+1)-x(i))*(y(i+1)+y(i))
                      ! ü;̾ûʬ
-              else
-                 if(i_bot==i_top)then
-                   ! ʬ֤ 1 ʻҤʤ, ʬԤä, exit ȴ.
-                    res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
-  &                     +0.5*(top-x(i))*(y_top+y(i))
-                    exit
-                 end if
-              end if
-           else
-              if(i==i_top)then
-                 res=res+0.5*(top-x(i))*(y_top+y(i))  ! ü;Τ
               else
                  res=res+0.5*(x(i+1)-x(i))*(y(i+1)+y(i))  ! ̾û
               end if
            end if
         end do
+        res=res+0.5*(top-x(i_top))*(y_top+y(i_top))  ! ü;Τ
+     else
+        do i=i_bot,i_top-1
+           if(i==i_bot)then
+              res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
+  &               +0.5*(x(i+1)-x(i))*(y(i+1)+y(i))
+                  ! ü;̾ûʬ
+           else
+              res=res+0.5*(x(i+1)-x(i))*(y(i+1)+y(i))  ! ̾û
+           end if
+        end do
+        res=res+0.5*(top-x(i_top))*(y_top+y(i_top))  ! ü;Τ
      end if
   else
      if(present(undeff))then
-        if(y(i_top)/=undeff.and.y(i_bot)/=undeff)then
+        if(y(i_bot)/=undeff)then
+           res=res+0.5*(x(i)-bot)*(y(i)+y_bot)  &
+  &            +0.5*(top-x(i))*(y_top+y(i))
+        else
            res=0.5*(top-bot)*(y_top+y_bot)
         end if
      else
-        res=0.5*(top-bot)*(y_top+y_bot)
+        res=res+0.5*(x(i_bot)-bot)*(y(i_bot)+y_bot)  &
+  &         +0.5*(top-x(i_bot))*(y_top+y(i_bot))
      end if
   end if
 
