@@ -111,6 +111,7 @@ real function precip_water( p, qv, undef )  ! Ĺ߿̤׻. ñ̤ [kg/m^2]
 ! ݤ狼ȼ, ٺɸʬΤǤϤʤ,
 ! ϳʿդ鵤ɸ֤ľʬ.
   use Algebra
+  use Phys_Const
   implicit none
   real, intent(in) :: p(:)  !  [Pa]
   real, intent(in) :: qv(size(p))  !  [kg/kg]
@@ -121,26 +122,20 @@ real function precip_water( p, qv, undef )  ! Ĺ߿̤׻. ñ̤ [kg/m^2]
 
   nx=size(p)
 
+!-- ʬΤ, ؤ
+  do i=1,nx
+     tmp_qv(i)=qv(nx-i+1)
+     tmp_p(i)=p(nx-i+1)
+  end do
+
   if(present(undef))then
-     do i=1,nx
-        if(p(i)==undef.or.qv(i)==undef)then
-           tmp_qv(i)=0.0
-           tmp_p(i)=tmp_p(i-1)
-        else
-           tmp_qv(i)=qv(i)
-           tmp_p(i)=p(i)
-        end if
-     end do
+     call rectangle_int( tmp_p, tmp_qv, tmp_p(1), tmp_p(nx), precip, undef )
   else
-     do i=1,nx
-        tmp_qv(i)=qv(i)
-        tmp_p(i)=p(i)
-     end do
+     call rectangle_int( tmp_p, tmp_qv, tmp_p(1), tmp_p(nx), precip )
   end if
 
-  call rectangle_int( tmp_p, tmp_qv, tmp_p(nx), tmp_p(1), precip )
-
-  precip_water=-precip/g
+write(*,*) g, precip, "hehehei"
+  precip_water=precip/g
 
   return
 end function
@@ -148,7 +143,7 @@ end function
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function CAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff )
+real function CAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, opt )
   use Thermo_Const
   use Thermo_Function
   use Algebra
@@ -160,46 +155,95 @@ real function CAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff )
   real, intent(in) :: tmp_t(size(tmp_p))  !  [K]
   real, intent(in) :: z_ref  ! ѡ夲 [m]
   real, intent(in), optional :: undeff
-  integer :: nx, i, j, iz_LFC, iz_LNB
+  integer, intent(in), optional :: opt  ! LNB ׻κݤΥץ (z_LNB )
+  integer :: nx, i, iz_LFC, iz_LNB, option
   real :: PLFC, TLFC, PLNB, ZLFC, ZLNB
   real :: tmp(size(tmp_p)), t_par(size(tmp_p))
+  real :: inv_tmp(size(tmp_p)), inv_p(size(tmp_p))
+  real :: undef
 
   t_par=0.0
 
   nx=size(tmp_p)
 
+  if(present(undeff))then
+     undef=undeff
+  else
+     undef=-999.0
+  end if
+
+  if(present(opt))then
+     option=opt
+  else
+     option=1
+  end if
+
 !-- LFC, LNB Ǥι, , Ϥη׻
 
   TLFC=T_LFC( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
-  ZLFC=z_LFC( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
-  ZLNB=z_LNB( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
-  call interpo_search_1d( tmp_z, ZLFC, iz_LFC, undeff )
-  call interpo_search_1d( tmp_z, ZLNB, iz_LNB, undeff )
-  call interpolation_1d( tmp_z(iz_LFC), tmp_z(iz_LFC+1), tmp_p(iz_LFC),  &
-  &    tmp_p(iz_LFC+1), ZLFC, PLFC )
-  call interpolation_1d( tmp_z(iz_LNB), tmp_z(iz_LNB+1), tmp_p(iz_LNB),  &
-  &    tmp_p(iz_LNB+1), ZLNB, PLNB )
+  ZLFC=z_LFC( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv, undeff=undef )
+  ZLNB=z_LNB( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv, opt=option, undeff=undef )
 
-  do i=1,nx
-     if(tmp_p(i)<=PLFC.and.tmp_p(i)>=PLNB)then
-        if(present(undeff))then
-           if(undeff==tmp_t(i))then
-              tmp(i)=undeff
+  if(ZLFC/=undef.and.ZLNB/=undef)then
+     call interpo_search_1d( tmp_z, ZLFC, iz_LFC, int(undef) )
+     call interpo_search_1d( tmp_z, ZLNB, iz_LNB, int(undef) )
+     if(iz_LFC>=nx)then
+        iz_LFC=int(undef)
+     end if
+     if(iz_LNB>=nx)then
+        iz_LNB=int(undef)
+     end if
+  else
+     iz_LFC=int(undef)
+     iz_LNB=int(undef)
+  end if
+
+  if(iz_LFC/=int(undef).and.iz_LNB/=int(undef))then
+
+  if(tmp_t(iz_LFC)/=undef.and.tmp_qv(iz_LFC)/=undef.and.tmp_p(iz_LFC)/=undef  &
+  &  .and.tmp_t(iz_LFC+1)/=undef.and.tmp_qv(iz_LFC+1)/=undef.and.tmp_p(iz_LFC+1)/=undef  &
+  &  .and.tmp_t(iz_LNB)/=undef.and.tmp_qv(iz_LNB)/=undef.and.tmp_p(iz_LNB)/=undef  &
+  &  .and.tmp_t(iz_LNB+1)/=undef.and.tmp_qv(iz_LNB+1)/=undef.and.tmp_p(iz_LNB+1)/=undef)then
+     call interpolation_1d( (/tmp_z(iz_LFC), tmp_z(iz_LFC+1)/),  &
+     &    (/tmp_p(iz_LFC), tmp_p(iz_LFC+1)/), ZLFC, PLFC )
+     call interpolation_1d( (/tmp_z(iz_LNB), tmp_z(iz_LNB+1)/),  &
+     &    (/tmp_p(iz_LNB), tmp_p(iz_LNB+1)/), ZLNB, PLNB )
+
+     do i=1,nx
+        if(iz_LFC<=i.and.i<=iz_LNB+1)then
+
+           if(undef==tmp_t(i))then
+              tmp(i)=undef
            else
               t_par(i)=moist_laps_temp( PLFC, TLFC, tmp_p(i) )
               tmp(i)=(t_par(i)-tmp_t(i))/tmp_p(i)
            end if
         else
-           t_par(i)=moist_laps_temp( PLFC, TLFC, tmp_p(i) )
-           tmp(i)=(t_par(i)-tmp_t(i))/tmp_p(i)
+           tmp(i)=undef
         end if
-     end if
-  end do
+     end do
 
-  call rectangle_int( tmp_p, tmp, PLNB, PLFC, CAPE, undeff )
-! PLNB Ƥ뤬, rectangle_int νԹǤʤäƤ.
-! ʬθ, \int^{PLNB}_{PLFC} ǹԤ.
-  CAPE=-CAPE*Rd
+!-- ʬΤ, ؤ
+     do i=1,nx
+        inv_tmp(i)=tmp(nx-i+1)
+        inv_p(i)=tmp_p(nx-i+1)
+     end do
+
+     call rectangle_int( inv_p, inv_tmp, PLNB, PLFC, CAPE, undef )
+
+     CAPE=CAPE*Rd
+
+  else
+
+     CAPE=undef
+
+  end if
+
+  else
+
+     CAPE=undef
+
+  end if
 
   return
 end function
@@ -219,56 +263,90 @@ real function CIN( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff )
   real, intent(in) :: tmp_t(size(tmp_p))  !  [K]
   real, intent(in) :: z_ref  ! ѡ夲 [m]
   real, intent(in), optional :: undeff
-  integer :: nx, i, j, iz_LFC, iz_ref, iz_LCL
+  integer :: nx, i, iz_LFC, iz_ref, iz_LCL
   real :: PLFC, TLFC, ZLFC, PLCL, ZLCL, TLCL, p_ref, T_ref, qv_ref
   real :: tmp(size(tmp_p)), t_par(size(tmp_p))
+  real :: inv_tmp(size(tmp_p)), inv_p(size(tmp_p))
+  real :: undef
 
   t_par=0.0
 
   nx=size(tmp_p)
 
+  if(present(undeff))then
+     undef=undeff
+  else
+     undef=-999.0
+  end if
+
 !-- z_ref Ȥ, LCL, LFC Ǥβ, , Ϥη׻
 
 !-- LFC ٤ѿ׻
   TLFC=T_LFC( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
   ZLFC=z_LFC( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
-  call interpo_search_1d( tmp_z, ZLFC, iz_LFC, undeff )
-  call interpolation_1d( tmp_z(iz_LFC), tmp_z(iz_LFC+1), tmp_p(iz_LFC),  &
-  &    tmp_p(iz_LFC+1), ZLFC, PLFC )
+
+  call interpo_search_1d( tmp_z, ZLFC, iz_LFC, int(undef) )
+  call interpo_search_1d( tmp_z, z_ref, iz_ref, int(undef) )
+
+  if(iz_LFC>=nx)then
+     iz_LFC=int(undef)
+  end if
+  if(iz_ref>=nx)then
+     iz_ref=int(undef)
+  end if
+
+  if(iz_LFC/=int(undef).and.iz_ref/=int(undef))then
+  if(tmp_t(iz_LFC)/=undef.and.tmp_qv(iz_LFC)/=undef.and.tmp_p(iz_LFC)/=undef  &
+  &  .and.tmp_t(iz_LFC+1)/=undef.and.tmp_qv(iz_LFC+1)/=undef.and.tmp_p(iz_LFC+1)/=undef  &
+  &  .and.tmp_t(iz_ref)/=undef.and.tmp_qv(iz_ref)/=undef.and.tmp_p(iz_ref)/=undef  &
+  &  .and.tmp_t(iz_ref+1)/=undef.and.tmp_qv(iz_ref+1)/=undef.and.tmp_p(iz_ref+1)/=undef)then
+
+     call interpolation_1d( (/tmp_z(iz_LFC), tmp_z(iz_LFC+1)/),  &
+     &    (/tmp_p(iz_LFC), tmp_p(iz_LFC+1)/), ZLFC, PLFC )
 !-- reference ٤ѿ׻
-  call interpo_search_1d( tmp_z, z_ref, iz_ref, undeff )
-  call interpolation_1d( tmp_z(iz_ref), tmp_z(iz_ref+1), tmp_p(iz_ref),  &
-  &    tmp_p(iz_ref+1), z_ref, p_ref )
-  call interpolation_1d( tmp_z(iz_ref), tmp_z(iz_ref+1), tmp_t(iz_ref),  &
-  &    tmp_t(iz_ref+1), z_ref, T_ref )
-  call interpolation_1d( tmp_z(iz_ref), tmp_z(iz_ref+1), tmp_qv(iz_ref),  &
-  &    tmp_qv(iz_ref+1), z_ref, qv_ref )
+     call interpolation_1d( (/tmp_z(iz_ref), tmp_z(iz_ref+1)/),  &
+     &    (/tmp_p(iz_ref), tmp_p(iz_ref+1)/), z_ref, p_ref )
+     call interpolation_1d( (/tmp_z(iz_ref), tmp_z(iz_ref+1)/),  &
+     &    (/tmp_t(iz_ref), tmp_t(iz_ref+1)/), z_ref, T_ref )
+     call interpolation_1d( (/tmp_z(iz_ref), tmp_z(iz_ref+1)/),  &
+     &    (/tmp_qv(iz_ref), tmp_qv(iz_ref+1)/), z_ref, qv_ref )
 !-- LCL ٤ѿ׻
-  TLCL=TqvP_2_TLCL( t_ref, qv_ref, p_ref )
-  ZLCL=z_LCL( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
-  call interpo_search_1d( tmp_z, ZLCL, iz_LCL, undeff )
-  call interpolation_1d( tmp_z(iz_LCL), tmp_z(iz_LCL+1), tmp_p(iz_LCL),  &
-  &    tmp_p(iz_LCL+1), ZLCL, PLCL )
+     TLCL=TqvP_2_TLCL( t_ref, qv_ref, p_ref )
+     ZLCL=z_LCL( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
+
+     call interpo_search_1d( tmp_z, ZLCL, iz_LCL, int(undef) )
+
+     if(iz_LCL>=nx)then
+        iz_LCL=int(undef)
+     end if
+
+     if(iz_LCL/=int(undef))then
+     if(tmp_t(iz_LCL)/=undef.and.tmp_qv(iz_LCL)/=undef.and.tmp_p(iz_LCL)/=undef  &
+  &  .and.tmp_t(iz_LCL+1)/=undef.and.tmp_qv(iz_LCL+1)/=undef.and.tmp_p(iz_LCL+1)/=undef)then
+
+        call interpolation_1d( (/tmp_z(iz_LCL), tmp_z(iz_LCL+1)/),  &
+        &    (/tmp_p(iz_LCL), tmp_p(iz_LCL+1)/), ZLCL, PLCL )
 
 ! Ϥβ٤ T ƤΤ, ޤ LCL ׻, ι
 ! ޤǤǮ, ʹߤ򼾽Ǯǥѡ벹٤׻.
 
-  do i=1,nx
-     if(tmp_z(i)>=z_ref.and.tmp_z(i)<=ZLCL)then
-        t_par(i)=T_ref-(g/Cpd)*(tmp_z(i)-z_ref)
-     else
-        t_par(i)=moist_laps_temp( PLCL, TLCL, tmp_p(i) )
-     end if
-     if(present(undeff))then
-        if(undeff==tmp_t(i))then
-           tmp(i)=undeff
-        else
-           tmp(i)=(t_par(i)-tmp_t(i))/tmp_p(i)
-        end if
-     else
-        tmp(i)=(t_par(i)-tmp_t(i))/tmp_p(i)
-     end if
-  end do
+        do i=1,nx
+           if(i>=iz_ref.and.i<=iz_LCL+1)then
+              t_par(i)=T_ref-(g/Cpd)*(tmp_z(i)-z_ref)
+           else
+              if(i>iz_LCL+1.and.i<=iz_LFC+1)then
+                 t_par(i)=moist_laps_temp( PLCL, TLCL, tmp_p(i) )
+              else
+                 t_par(i)=undef
+              end if
+           end if
+
+           if(undef==tmp_t(i).or.t_par(i)==undef)then
+              tmp(i)=undef
+           else
+              tmp(i)=(t_par(i)-tmp_t(i))/tmp_p(i)
+           end if
+        end do
 
 !do i=1,nx
 !   if(tmp_p(i)<p_ref.and.tmp_p(i)>PLFC)then
@@ -281,10 +359,39 @@ real function CIN( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff )
 !   end if
 !end do
 
-  call rectangle_int( tmp_p, tmp, PLFC, p_ref, CIN, undeff )
-! PLNB Ƥ뤬, rectangle_int νԹǤʤäƤ.
-! ʬθ, \int^{PLNB}_{PLFC} ǹԤ.
-  CIN=-CIN*Rd
+!-- ʬΤ, ؤ
+        do i=1,nx
+           inv_tmp(i)=tmp(nx-i+1)
+           inv_p(i)=tmp_p(nx-i+1)
+        end do
+
+        call rectangle_int( inv_p, inv_tmp, PLFC, p_ref, CIN, undef )
+
+        CIN=CIN*Rd
+
+     else
+
+        CIN=undef
+
+     end if
+
+     else
+
+        CIN=undef
+
+     end if
+
+  else
+
+     CIN=undef
+
+  end if
+
+  else
+
+     CIN=undef
+
+  end if
 
   return
 end function
@@ -292,7 +399,7 @@ end function
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function z_LCL( z_ref, z, temp, pres, qv )  ! LCL ٤׻.
+real function z_LCL( z_ref, z, temp, pres, qv, undeff )  ! LCL ٤׻.
 ! ѡ뤬 z_ref ǮǾ徺, β٤ãȤι٤
 ! LCL ٤ȤʤΤ, z_ref 鴥ǮΨǰʲΤ褦˷׻.
 ! LCL ٤ z_LCL Ȥ, ѡδ٤ z_ref Ȥ,
@@ -302,6 +409,7 @@ real function z_LCL( z_ref, z, temp, pres, qv )  ! LCL ٤׻.
 ! $z_LCL=z_ref+\frac{C_p}{g} (T_ref-T_LCL)$
 ! Ȥʤ.
   use Thermo_Const
+  use Thermo_Function
   use Phys_Const
   use Statistics
   implicit none
@@ -310,20 +418,48 @@ real function z_LCL( z_ref, z, temp, pres, qv )  ! LCL ٤׻.
   real, intent(in) :: temp(size(z))  !  [K]
   real, intent(in) :: pres(size(z))  !  [Pa]
   real, intent(in) :: qv(size(z))  !  [kg/kg]
-  integer :: i, j, k, nz, iz_ref
+  real ,intent(in), optional :: undeff
+  integer :: iz_ref
   real :: TLCL
   real :: T_ref, P_ref, qv_ref
+  real :: undef
 
-  call interpo_search_1d( z, z_ref, iz_ref )
-  call interpolation_1d( z(iz_ref), z(iz_ref), temp(iz_ref),  &
-  &    temp(iz_ref+1), z_ref, T_ref )
-  call interpolation_1d( z(iz_ref), z(iz_ref), pres(iz_ref),  &
-  &    pres(iz_ref+1), z_ref, P_ref )
-  call interpolation_1d( z(iz_ref), z(iz_ref), qv(iz_ref),  &
-  &    qv(iz_ref+1), z_ref, qv_ref )
+  if(present(undeff))then
+     undef=undeff
+  else
+     undef=-999.0
+  end if
 
-  TLCL=TqvP_2_TLCL( T_ref, P_ref, qv_ref )  ! z_ref Υѡ LCL Ǥβ.
-  z_LCL=z_ref+(Cpd/g)*(T_ref-TLCL)
+  call interpo_search_1d( z, z_ref, iz_ref, int(undef) )
+
+  if(iz_ref>=size(z))then
+     iz_ref=int(undef)
+  end if
+
+  if(iz_ref/=int(undef))then
+  if(temp(iz_ref)/=undef.and.qv(iz_ref)/=undef.and.pres(iz_ref)/=undef  &
+  &  .and.temp(iz_ref+1)/=undef.and.qv(iz_ref+1)/=undef.and.pres(iz_ref+1)/=undef)then
+     call interpolation_1d( (/z(iz_ref), z(iz_ref+1)/),  &
+     &    (/temp(iz_ref), temp(iz_ref+1)/), z_ref, T_ref )
+     call interpolation_1d( (/z(iz_ref), z(iz_ref+1)/),  &
+     &    (/pres(iz_ref), pres(iz_ref+1)/), z_ref, P_ref )
+     call interpolation_1d( (/z(iz_ref), z(iz_ref+1)/),  &
+     &    (/qv(iz_ref), qv(iz_ref+1)/), z_ref, qv_ref )
+
+     TLCL=TqvP_2_TLCL( T_ref, qv_ref, P_ref )  ! z_ref Υѡ LCL Ǥβ.
+     z_LCL=z_ref+(Cpd/g)*(T_ref-TLCL)
+
+  else
+
+     z_LCL=undef
+
+  end if
+
+  else
+
+     z_LCL=undef
+
+  end if
 
   return
 end function
@@ -331,7 +467,7 @@ end function
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function z_LFC( z_ref, z, temp, pres, qv )  ! LFC ٤׻.
+real function z_LFC( z_ref, z, temp, pres, qv, undeff )  ! LFC ٤׻.
 ! ٤Ǥ̤徺Ȥ, Ϥ˰̤Ȱפ٤
 ! LFC Ǥ. 
 ! Ǥ, Ϥ˰̤ȸ򺹤 2 , 
@@ -344,11 +480,20 @@ real function z_LFC( z_ref, z, temp, pres, qv )  ! LFC ٤׻.
   real, intent(in) :: temp(size(z))  !  [K]
   real, intent(in) :: pres(size(z))  !  [Pa]
   real, intent(in) :: qv(size(z))  !  [kg/kg]
-  integer :: i, j, k, nz, iz, iept
+  real, intent(in), optional :: undeff
+  integer :: i, nz, iz, iept
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
+  real :: undef
 
   nz=size(z)
+  iept=0
+
+  if(present(undeff))then
+     undef=undeff
+  else
+     undef=-999.0
+  end if
 
 !-- ľ˰̤׻.
   do i=1,nz
@@ -356,25 +501,49 @@ real function z_LFC( z_ref, z, temp, pres, qv )  ! LFC ٤׻.
   end do
 
 !-- z_ref Ǥ̤׻.
-  call interpo_search_1d( z, z_ref, iz )
+  call interpo_search_1d( z, z_ref, iz, int(undef) )
+
+  if(iz>=nz)then
+     iz=int(undef)
+  end if
+
+  if(iz/=int(undef))then
+  if(temp(iz)/=undef.and.qv(iz)/=undef.and.pres(iz)/=undef  &
+  &  .and.temp(iz+1)/=undef.and.qv(iz+1)/=undef.and.pres(iz+1)/=undef)then
 !-- iz  iz+1 Ǥ̤׻.
-  eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
-  eptiz1=thetae_Bolton( temp(iz+1), qv(iz+1), pres(iz+1) )
+     eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
+     eptiz1=thetae_Bolton( temp(iz+1), qv(iz+1), pres(iz+1) )
 !-- δ֤̤
-  call interpolation_1d( z(iz), z(iz+1), eptiz, eptiz1, z_ref, ept_ref )
+     call interpolation_1d( (/z(iz), z(iz+1)/), (/eptiz, eptiz1/), z_ref, ept_ref )
 !-- z_ref ι٤ ept_ref ˰̤δط׻.
-  do i=iz+1,nz
-     if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
-        iept=i
-        exit
-     end if
-  end do
+     do i=iz+1,nz-1
+        if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
+           iept=i
+           exit
+        end if
+     end do
 !-- ѡ̤Ķ˰̤ȵžľι٤ iept Ǥ.
 !-- , iept  iept+1 ˰̤٤ޤ.
-  call interpolation_1d( sept(iept), sept(iept+1), z(iept), z(iept+1),  &
-  &                      ept_ref, z_sol )
+     if(iept>0.and.iept<nz)then
+        call interpolation_1d( (/sept(iept), sept(iept+1)/),  &
+  &                            (/z(iept), z(iept+1)/), ept_ref, z_sol )
 
-  z_LFC=z_sol
+        z_LFC=z_sol
+     else
+        z_LFC=undef
+     end if
+
+  else
+
+     z_LFC=undef
+
+  end if
+
+  else
+
+     z_LFC=undef
+
+  end if
 
   return
 end function
@@ -382,7 +551,7 @@ end function
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function z_LNB( z_ref, z, temp, pres, qv, opt )  ! LNB ٤׻.
+real function z_LNB( z_ref, z, temp, pres, qv, opt, undeff )  ! LNB ٤׻.
 ! LFC ٰʾǺƤӴĶ˰̤ȸ٤ LNB Ǥ.
 ! , ºݤ¬ǡǤ, ¬ưˤ, LFC ľ LNB 
 ! ã礬.
@@ -400,11 +569,20 @@ real function z_LNB( z_ref, z, temp, pres, qv, opt )  ! LNB ٤׻.
   real, intent(in) :: pres(size(z))  !  [Pa]
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   integer, intent(in), optional :: opt  ! ׻ˡΥץ
-  integer :: i, j, k, nz, iz, iept, counter
+  real, intent(in), optional :: undeff
+  integer :: i, nz, iz, iept, counter
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
+  real :: undef
 
   nz=size(z)
+  iept=0
+
+  if(present(undeff))then
+     undef=undeff
+  else
+     undef=-999.0
+  end if
 
 !-- ľ˰̤׻.
   do i=1,nz
@@ -412,26 +590,45 @@ real function z_LNB( z_ref, z, temp, pres, qv, opt )  ! LNB ٤׻.
   end do
 
 !-- z_ref Ǥ̤׻.
-  call interpo_search_1d( z, z_ref, iz )
+  call interpo_search_1d( z, z_ref, iz, int(undef) )
+
+  if(iz>=nz)then
+     iz=int(undef)
+  end if
+
+  if(iz/=int(undef))then
+  if(temp(iz)/=undef.and.qv(iz)/=undef.and.pres(iz)/=undef  &
+  &  .and.temp(iz+1)/=undef.and.qv(iz+1)/=undef.and.pres(iz+1)/=undef)then
 !-- iz  iz+1 Ǥ̤׻.
-  eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
-  eptiz1=thetae_Bolton( temp(iz+1), qv(iz+1), pres(iz+1) )
+     eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
+     eptiz1=thetae_Bolton( temp(iz+1), qv(iz+1), pres(iz+1) )
 !-- δ֤̤
-  call interpolation_1d( z(iz), z(iz+1), eptiz, eptiz1, z_ref, ept_ref )
+     call interpolation_1d( (/z(iz), z(iz+1)/), (/eptiz, eptiz1/), z_ref, ept_ref )
 
-  counter=0
+     counter=0
 
-  if(present(opt))then
-     if(opt==2)then
-        do i=nz,iz+1,-1  ! 夫鲼˲.
-           if((sept(i)-ept_ref)*(sept(i-1)-ept_ref)<0.0)then
-              iept=i-1  ! 夫鲼˲ƤΤ, 1 ǲΥǡ iept.
-              exit
-           end if
-        end do
-     else
+     if(present(opt))then
+        if(opt==2)then
+           do i=nz,iz+1,-1  ! 夫鲼˲.
+              if((sept(i)-ept_ref)*(sept(i-1)-ept_ref)<0.0)then
+                 iept=i-1  ! 夫鲼˲ƤΤ, 1 ǲΥǡ iept.
+                 exit
+              end if
+           end do
+        else
 !-- z_ref ι٤ ept_ref ˰̤δط׻.
-        do i=iz+1,nz
+           do i=iz+1,nz-1
+              if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
+                 counter=counter+1
+                 if(counter==2)then
+                    iept=i
+                    exit
+                 end if
+              end if
+           end do
+        end if
+     else
+        do i=iz+1,nz-1
            if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
               counter=counter+1
               if(counter==2)then
@@ -441,23 +638,28 @@ real function z_LNB( z_ref, z, temp, pres, qv, opt )  ! LNB ٤׻.
            end if
         end do
      end if
-  else
-     do i=iz+1,nz
-        if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
-           counter=counter+1
-           if(counter==2)then
-              iept=i
-              exit
-           end if
-        end if
-     end do
-  end if
 !-- ѡ̤Ķ˰̤ȵžľι٤ iept Ǥ.
 !-- , iept  iept+1 ˰̤٤ޤ.
-  call interpolation_1d( sept(iept), sept(iept+1), z(iept), z(iept+1),  &
-  &                      ept_ref, z_sol )
+     if(iept>0.and.iept<nz)then
+        call interpolation_1d( (/sept(iept), sept(iept+1)/),  &
+  &                            (/z(iept), z(iept+1)/), ept_ref, z_sol )
 
-  z_LNB=z_sol
+        z_LNB=z_sol
+     else
+        z_LNB=undef
+     end if
+
+  else
+
+     z_LNB=undef
+
+  end if
+
+  else
+
+     z_LNB=undef
+
+  end if
 
   return
 end function
@@ -465,7 +667,7 @@ end function
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function T_LFC( z_ref, z, temp, pres, qv )  ! LFC ٤Ǥβ٤׻.
+real function T_LFC( z_ref, z, temp, pres, qv, undeff )  ! LFC ٤Ǥβ٤׻.
 ! ٤Ǥ̤徺Ȥ, Ϥ˰̤Ȱפ٤
 ! LFC Ǥ. 
 ! Ǥ, Ϥ˰̤ȸ򺹤 2 , 
@@ -478,11 +680,20 @@ real function T_LFC( z_ref, z, temp, pres, qv )  ! LFC ٤Ǥβ٤׻.
   real, intent(in) :: temp(size(z))  !  [K]
   real, intent(in) :: pres(size(z))  !  [Pa]
   real, intent(in) :: qv(size(z))  !  [kg/kg]
-  integer :: i, j, k, nz, iz, iept
+  real, intent(in), optional :: undeff
+  integer :: i, nz, iz, iept
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
+  real :: undef
 
   nz=size(z)
+  iept=0
+
+  if(present(undeff))then
+     undef=undeff
+  else
+     undef=-999.0
+  end if
 
 !-- ľ˰̤׻.
   do i=1,nz
@@ -490,25 +701,48 @@ real function T_LFC( z_ref, z, temp, pres, qv )  ! LFC ٤Ǥβ٤׻.
   end do
 
 !-- z_ref Ǥ̤׻.
-  call interpo_search_1d( z, z_ref, iz )
+  call interpo_search_1d( z, z_ref, iz, int(undef) )
+  if(iz>=nz)then
+     iz=int(undef)
+  end if
+
 !-- iz  iz+1 Ǥ̤׻.
-  eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
-  eptiz1=thetae_Bolton( temp(iz+1), qv(iz+1), pres(iz+1) )
+  if(iz/=int(undef))then
+  if(temp(iz)/=undef.and.qv(iz)/=undef.and.pres(iz)/=undef  &
+  &  .and.temp(iz+1)/=undef.and.qv(iz+1)/=undef.and.pres(iz+1)/=undef)then
+     eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
+     eptiz1=thetae_Bolton( temp(iz+1), qv(iz+1), pres(iz+1) )
 !-- δ֤̤
-  call interpolation_1d( z(iz), z(iz+1), eptiz, eptiz1, z_ref, ept_ref )
+     call interpolation_1d( (/z(iz), z(iz+1)/), (/eptiz, eptiz1/), z_ref, ept_ref )
 !-- z_ref ι٤ ept_ref ˰̤δط׻.
-  do i=iz+1,nz
-     if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
-        iept=i
-        exit
-     end if
-  end do
+     do i=iz+1,nz-1
+        if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
+           iept=i
+           exit
+        end if
+     end do
 !-- ѡ̤Ķ˰̤ȵžľι٤ iept Ǥ.
 !-- , iept  iept+1 ˰̤٤ޤ.
-  call interpolation_1d( sept(iept), sept(iept+1), temp(iept), temp(iept+1),  &
-  &                      ept_ref, z_sol )
+     if(iept>0.and.iept<nz)then
+        call interpolation_1d( (/sept(iept), sept(iept+1)/),  &
+     &                         (/temp(iept), temp(iept+1)/), ept_ref, z_sol )
 
-  T_LFC=z_sol
+        T_LFC=z_sol
+     else
+        T_LFC=undef
+     end if
+
+  else
+
+     T_LFC=undef
+
+  end if
+
+  else
+
+     T_LFC=undef
+
+  end if
 
   return
 end function
@@ -516,7 +750,7 @@ end function
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function T_LNB( z_ref, z, temp, pres, qv, opt )  ! LNB ٤Ǥβ٤׻.
+real function T_LNB( z_ref, z, temp, pres, qv, opt, undeff )  ! LNB ٤Ǥβ٤׻.
 ! LFC ٰʾǺƤӴĶ˰̤ȸ٤ LNB Ǥ.
 ! , ºݤ¬ǡǤ, ¬ưˤ, LFC ľ LNB 
 ! ã礬.
@@ -534,11 +768,20 @@ real function T_LNB( z_ref, z, temp, pres, qv, opt )  ! LNB ٤Ǥβ٤׻.
   real, intent(in) :: pres(size(z))  !  [Pa]
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   integer, intent(in), optional :: opt  ! ׻ˡΥץ
-  integer :: i, j, k, nz, iz, iept, counter
+  real, intent(in), optional :: undeff
+  integer :: i, nz, iz, iept, counter
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
+  real :: undef
 
   nz=size(z)
+  iept=0
+
+  if(present(undeff))then
+     undef=undeff
+  else
+     undef=-999.0
+  end if
 
 !-- ľ˰̤׻.
   do i=1,nz
@@ -546,26 +789,44 @@ real function T_LNB( z_ref, z, temp, pres, qv, opt )  ! LNB ٤Ǥβ٤׻.
   end do
 
 !-- z_ref Ǥ̤׻.
-  call interpo_search_1d( z, z_ref, iz )
+  call interpo_search_1d( z, z_ref, iz, int(undef) )
+  if(iz>=nz)then
+     iz=int(undef)
+  end if
+
 !-- iz  iz+1 Ǥ̤׻.
-  eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
-  eptiz1=thetae_Bolton( temp(iz+1), qv(iz+1), pres(iz+1) )
+  if(iz/=int(undef))then
+  if(temp(iz)/=undef.and.qv(iz)/=undef.and.pres(iz)/=undef  &
+  &  .and.temp(iz+1)/=undef.and.qv(iz+1)/=undef.and.pres(iz+1)/=undef)then
+     eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
+     eptiz1=thetae_Bolton( temp(iz+1), qv(iz+1), pres(iz+1) )
 !-- δ֤̤
-  call interpolation_1d( z(iz), z(iz+1), eptiz, eptiz1, z_ref, ept_ref )
+     call interpolation_1d( (/z(iz), z(iz+1)/), (/eptiz, eptiz1/), z_ref, ept_ref )
 
-  counter=0
+     counter=0
 
-  if(present(opt))then
-     if(opt==2)then
-        do i=nz,iz+1,-1  ! 夫鲼˲.
-           if((sept(i)-ept_ref)*(sept(i-1)-ept_ref)<0.0)then
-              iept=i-1  ! 夫鲼˲ƤΤ, 1 ǲΥǡ iept.
-              exit
-           end if
-        end do
-     else
+     if(present(opt))then
+        if(opt==2)then
+           do i=nz,iz+1,-1  ! 夫鲼˲.
+              if((sept(i)-ept_ref)*(sept(i-1)-ept_ref)<0.0)then
+                 iept=i-1  ! 夫鲼˲ƤΤ, 1 ǲΥǡ iept.
+                 exit
+              end if
+           end do
+        else
 !-- z_ref ι٤ ept_ref ˰̤δط׻.
-        do i=iz+1,nz
+           do i=iz+1,nz-1
+              if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
+                 counter=counter+1
+                 if(counter==2)then
+                    iept=i
+                    exit
+                 end if
+              end if
+           end do
+        end if
+     else
+        do i=iz+1,nz-1
            if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
               counter=counter+1
               if(counter==2)then
@@ -575,23 +836,28 @@ real function T_LNB( z_ref, z, temp, pres, qv, opt )  ! LNB ٤Ǥβ٤׻.
            end if
         end do
      end if
-  else
-     do i=iz+1,nz
-        if((sept(i)-ept_ref)*(sept(i+1)-ept_ref)<0.0)then
-           counter=counter+1
-           if(counter==2)then
-              iept=i
-              exit
-           end if
-        end if
-     end do
-  end if
 !-- ѡ̤Ķ˰̤ȵžľι٤ iept Ǥ.
 !-- , iept  iept+1 ˰̤٤ޤ.
-  call interpolation_1d( sept(iept), sept(iept+1), temp(iept), temp(iept+1),  &
-  &                      ept_ref, z_sol )
+     if(iept>0.and.iept<nz)then
+        call interpolation_1d( (/sept(iept), sept(iept+1)/),  &
+     &                         (/temp(iept), temp(iept+1)/), ept_ref, z_sol )
 
-  T_LNB=z_sol
+        T_LNB=z_sol
+     else
+        T_LNB=undef
+     end if
+
+  else
+
+     T_LNB=undef
+
+  end if
+
+  else
+
+     T_LNB=undef
+
+  end if
 
   return
 end function
